#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-07


import json
from datetime import datetime

from marshmallow import Schema, \
    validates, \
    ValidationError, \
    post_load
from marshmallow.fields import String, \
    DateTime, \
    Integer, \
    Float

from gagospecs.v1 import \
    CoordinateSystemValues, \
    CompressionValues, \
    GeometryTypeValues
from gagospecs.v1.base.load_module import \
    create_module
from gagospecs.v1.base.model import \
    GMDModel, \
    CategoryValues, \
    BiasValues, \
    ProductLevelValues


def post_load_object(obj, **kwargs):
    for k, v in kwargs.items():
        name: str = k
        if hasattr(obj, name):
            setattr(obj, name, v)

    return obj


class GMDSchema(Schema):
    """
    GMD佳格元数据模式
    """
    version = String(default='1.0', required=True)
    name = String(required=True, allow_none=False)
    description = String(required=True)
    level = String(required=True)
    category = String(required=True)
    product_type = String(required=True)
    file_type = String(required=True)
    production_datetime = DateTime(required=True, format='%Y-%m-%dT%H:%M:%S%z')
    producer = String(required=False)
    qc = String(required=False)

    @validates('version')
    def validate_version(self, value):
        if value != '1.0':
            raise ValidationError('version字段必须为1.0')

    @validates('level')
    def validate_level(self, value):
        if value not in ProductLevelValues.get_value_set():
            raise ValidationError(
                'level字段必须为level1,level2或level3'
            )

    @validates('production_datetime')
    def validate_production_datetime(self, value):
        if value is None or not isinstance(value, datetime):
            raise ValidationError(
                'production_date必须为有效时间'
            )
        value: datetime = value
        if value.tzinfo is None:
            raise ValidationError(
                'production_date必须包含时区信息'
            )

    @post_load
    def make_object(self, data: dict):
        return post_load_object(GMDModel(), **data)


class SpatialGMDSchema(GMDSchema):
    """
    空间数据产品GMD模式
    """
    coordinate_system = String(required=True, allow_none=False)
    extent = String(required=True, allow_none=False)
    bias = String(required=True, allow_none=False)

    @validates('coordinate_system')
    def validate_coordinate_system(self, value):
        if value not in CoordinateSystemValues.get_value_set():
            raise ValidationError(
                'coordinate_system字段必须为"EPSG:3857"或"EPSG:4326"'
            )

    @validates('extent')
    def validate_extent(self, value):
        try:
            extent_value: dict = json.loads(value)
            if not isinstance(extent_value, list):
                raise ValidationError('extent字段必须为坐标数组')
            if len(extent_value) != 4:
                raise ValidationError('extent字段坐标数组必须为4对坐标值')
            for coordinates in extent_value:
                if not isinstance(coordinates, list) or \
                                len(coordinates) != 2:
                    raise ValidationError('extent字段坐标必须为坐标对')
        except json.JSONDecodeError:
            raise ValidationError('extent字段必须为合法JSON字符串')

    @validates('bias')
    def validate_bias(self, value):
        if value not in BiasValues.get_value_set():
            raise ValidationError('bias字段必须为"NA"或"CGJ-02"')


class RasterGMDSchema(SpatialGMDSchema):
    """
    栅格数据GMD头文件模式
    """
    no_data = Integer(required=True, allow_none=False)
    data_type = String(required=True, allow_none=False)

    @validates('category')
    def validate_category(self, value):
        if value != CategoryValues.Raster:
            raise ValidationError('category字段值必须为Raster')


class FloatRasterGMDSchema(RasterGMDSchema):
    """
    浮点型栅格数据产品GMD头文件模式
    """
    scale_factor = Float(required=True)
    linear_factor = Float(required=True)

    @validates('scale_factor')
    def validate_scale_factor(self, value):
        if value <= 0:
            raise ValidationError('scale_factor字段必须大于0')


class GeoTIFFGMDSchema(Schema):
    """
    GeoTIFF头文件模式
    """
    compression = String(required=True)

    @validates('compression')
    def validate_compression(self, value):
        if value not in CompressionValues.get_value_set():
            raise ValidationError('compression字段只能为"NA"或"LZW"')


class VectorGMDSchema(SpatialGMDSchema):
    """
    矢量数据GMD头文件模式
    """
    geometry_type = String(required=True)

    @validates('geometry_type')
    def validate_geometry_type(self, value):
        if value not in GeometryTypeValues.get_value_set():
            raise ValidationError(
                'geometry_type字段只能为"Point", "Polyline"或"Polygon"'
            )


def create_schema(
        product_level: str,
        product_type: str,
) -> GMDSchema or None:
    """
    create a schema object
    :param product_level:
    :param product_type:
    :return:
    """
    m = create_module(
        'schemas',
        product_level,
        product_type
    )
    if m:
        if hasattr(m, 'main'):
            return m.main()
    return None



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-25


from gagospecs.v1.base.model import GMDModel, \
    DataProperties, \
    GMDProperties, \
    ProductLevelValues


class Level2GMDModel(GMDModel):
    def __init__(self):
        super(Level2GMDModel, self).__init__()
        self.level = ProductLevelValues.Level2


class Level2RasterGMDModel(Level2GMDModel):
    """
    二级栅格产品元数据模型
    """
    def __init__(self):
        super(Level2RasterGMDModel, self).__init__()
        self.columns: int = 0
        self.rows: int = 0
        self.resolution: float = -1

    def _validate(
            self,
            data_properties: DataProperties,
            errors: list
    ):
        if data_properties.columns \
                and self.columns != data_properties.columns:
            errors.append('columns')
        if data_properties.rows \
                and self.rows != data_properties.rows:
            errors.append('rows')
        if data_properties.resolution \
                and self.resolution != data_properties.resolution:
            errors.append('resolution')

    def update(
            self,
            data_properties: DataProperties,
            gmd_properties: GMDProperties
    ):
        super().update(
            data_properties,
            gmd_properties
        )
        if data_properties.columns:
            self.columns = data_properties.columns
        if data_properties.rows:
            self.rows = data_properties.rows
        if data_properties.resolution:
            self.resolution = data_properties.resolution



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-25


from marshmallow import ValidationError, \
    validates
from marshmallow.fields import Integer, \
    Float

from gagospecs.v1.base.model import ProductLevelValues
from gagospecs.v1.base.schema import GMDSchema


class Level2GMDSchema(GMDSchema):
    """
    三级产品模式
    """
    @validates('level')
    def validate_level(self, value):
        if value != ProductLevelValues.Level2:
            raise ValidationError(
                '二级产品level字段必须为level2'
            )


class Level2RasterSchema(Level2GMDSchema):
    """
    二级栅格产品模式
    """
    columns = Integer(required=True, allow_none=False)
    rows = Integer(required=True, allow_none=False)
    resolution = Float(required=True, allow_none=False)

    @validates('columns')
    def validate_columns(self, value):
        if value <= 0:
            raise ValidationError('columns字段必须大于0')

    @validates('rows')
    def validate_rows(self, value):
        if value <= 0:
            raise ValidationError('rows字段必须大于0')

    @validates('resolution')
    def validate_resolution(self, value):
        if value <= 0:
            raise ValidationError('resolution字段必须大于0')



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 018-10-17


from marshmallow import \
    ValidationError, \
    validates, \
    post_load

from gagospecs.v1 import ProductLevelValues, \
    ProductTypeValues, \
    FileTypeValues, \
    RasterDataTypeValues, \
    CompressionValues
from gagospecs.v1.base.schema import \
    FloatRasterGMDSchema, \
    GeoTIFFGMDSchema
from gagospecs.v1.base.schema import post_load_object
from gagospecs.v1.products.level2.models.NDVI import NDVIModel
from gagospecs.v1.products.level2.schemas.general import Level2RasterSchema


class NDVISchema(
    FloatRasterGMDSchema,
    GeoTIFFGMDSchema,
    Level2RasterSchema,
):

    @validates('file_type')
    def validate_file_type(self, value):
        if value != FileTypeValues.GeoTIFF:
            raise ValidationError('二级NDVI栅格产品file_type字段必须为"GeoTIFF"')

    @validates('product_type')
    def validate_product_type(self, value):
        if value != ProductTypeValues.NDVI:
            raise ValidationError('二级NDVI栅格产品的product_type字段必须为"NDVI"')

    @validates('data_type')
    def validate_data_type(self, value):
        if value != RasterDataTypeValues.Byte:
            raise ValidationError('二级NDVI栅格产品data_type字段必须为"Byte"')

    @validates('no_data')
    def validate_no_data(self, value):
        if value != 255:
            raise ValidationError('二级NDVI栅格产品no_data字段必须为"255"')

    @validates('scale_factor')
    def validate_scale_factor(self, value):
        if value != 0.0078125:
            raise ValidationError('二级NDVI栅格产品scale_factor字段必须为"0.0078125"')

    @validates('linear_factor')
    def validate_linear_factor(self, value):
        if value != -127:
            raise ValidationError('二级NDVI栅格产品linear_factor字段必须为"-127"')

    @validates('compression')
    def validate_compression(self, value):
        if value != CompressionValues.LZW:
            raise ValidationError('二级NDVI栅格产品compression字段必须为"LZW"')

    @post_load
    def make_object(self, data: dict):
        return post_load_object(NDVIModel(), **data)


def main():
    return NDVISchema()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-26


from gagospecs.v1.base.load_model import \
    load_level_and_type_from_gmd_str
from gagospecs.v1.base.model import DataProperties, GMDProperties
from gagospecs.v1.base.model import create_data_properties, \
    create_gmd_properties


def get_data_properties_from_level_and_type(
        product_level: str,
        product_type: str
) -> DataProperties:
    return create_data_properties(
        product_level,
        product_type
    )


def get_data_properties(
        gmd_str: str,
) -> DataProperties:
    """
    get a DataProperties object from a gmd str
    :param gmd_str: gmd str from gmd file or url
    :return: DataProperties object
    """
    product_level, product_type, errors = \
        load_level_and_type_from_gmd_str(
            gmd_str
        )
    if errors:
        return errors
    return get_data_properties_from_level_and_type(
        product_level,
        product_type
    )


def get_gmd_properties_from_level_and_type(
        product_level: str,
        product_type: str
) -> GMDProperties:
    return create_gmd_properties(
        product_level,
        product_type
    )


def get_gmd_properties(
        gmd_str: str,
) -> GMDProperties:
    product_level, product_type, errors = \
        load_level_and_type_from_gmd_str(
            gmd_str
        )
    if errors:
        return errors
    return get_gmd_properties_from_level_and_type(
        product_level,
        product_type
    )



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-19


from json import loads, JSONDecodeError

from gagospecs.v1.base.model import GMDModel
from gagospecs.v1.base.schema import GMDSchema, \
    create_schema


def load_level_and_type_from_gmd(
        gmd_dict: dict
) -> (str, str, dict):
    """
    Load product level and type from a gmd dict object,
    which is from gmd file or url.
    :param gmd_dict: gmd str object
    :return: product level, product type and errors dict
    if there are some errors in gmd_dict
    """
    gmd_schema: GMDSchema = GMDSchema()
    gmd_model, errors = gmd_schema.load(
        gmd_dict,
        False
    )
    if errors:
        return '', '', errors
    gmd_model: GMDModel = gmd_model
    return gmd_model.level, gmd_model.product_type, {}


def load_level_and_type_from_gmd_str(
        gmd_str: str
) -> (str, str, dict):
    """
    Load product level and type from gmd str object
    :param gmd_str:
    :return:
    """
    try:
        gmd_dict: dict = loads(gmd_str)
    except JSONDecodeError:
        return '', '', {
            'GMD格式': 'GMD格式错误'
        }
    return load_level_and_type_from_gmd(gmd_dict)


def load_model_from_gmd(
        gmd_dict: dict
) -> (GMDModel, dict):
    """
    Load a gmd model from a gmd dict object, which is from gmd file or url.
    :param gmd_dict: gmd object dict
    :return: (gmd model, {}) if there is no error in gmd_dict,
    or (None, errors) if there are some errors in a error dict
    """
    gmd_schema: GMDSchema = GMDSchema()
    gmd_model, errors = gmd_schema.load(
        gmd_dict,
        False
    )
    if errors:
        raise ValueError('给定的元数据格式或内容错误，无法加载')

    gmd_model: GMDModel = gmd_model
    real_schema: GMDSchema = create_schema(
        gmd_model.level,
        gmd_model.product_type
    )
    real_model, errors = real_schema.load(
        gmd_dict
    )
    if errors:
        return None, errors
    return real_model, {}



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-18


from json import loads, JSONDecodeError

from gagospecs.v1.base.field_mappings import FIELD_MAPPINGS
from gagospecs.v1.base.load_model import \
    load_model_from_gmd
from gagospecs.v1.base.model import GMDModel, \
    DataProperties


def validate_data(
        data_properties: DataProperties,
        gmd_str: str,
) -> dict:
    """
    validate if data is correct and if data matches mata data
    :param gmd_str: gago meta data dict object, from json str
    :param data_properties: properties of data itself(GeoTIFF, GeoJSON...)
    :return:
    """
    try:
        gmd_dict: dict = loads(gmd_str)
    except JSONDecodeError:
        return {'gmd内容': 'Gago Meta Data 内容错误'}

    gmd_model, errors_dict = load_model_from_gmd(
        gmd_dict
    )

    if errors_dict:
        return errors_dict
    gmd_model: GMDModel = gmd_model
    errors_list: list = gmd_model.validate(
        data_properties
    )
    errors_result_dict = dict()
    for error in errors_list:
        if error in FIELD_MAPPINGS:
            errors_result_dict[
                FIELD_MAPPINGS[error][0] + '/' + error
            ]\
                = '数据值域与元数据不符'
    return errors_result_dict



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-25


from gagospecs.v1.base.model import GMDModel, \
    DataProperties, \
    GMDProperties, \
    ProductLevelValues


class Level3Model(GMDModel):
    def __init__(self):
        super(Level3Model, self).__init__()
        self.level = ProductLevelValues.Level3


class Level3RasterModel(Level3Model):
    """
    三级栅格产品GMD模型
    """
    def __init__(self):
        super(Level3RasterModel, self).__init__()
        self.min_tile_level = -1
        self.max_tile_level = -1

    def _validate(
            self,
            data_properties: DataProperties,
            errors: list
    ):
        super()._validate(
            data_properties,
            errors
        )
        if self.min_tile_level != \
                data_properties.min_tile_level:
            errors.append('min_tile_level')
        if self.max_tile_level != \
                data_properties.max_tile_level:
            errors.append('max_tile_level')

    def update(
            self,
            data_properties: DataProperties,
            gmd_properties: GMDProperties
    ):
        super().update(
            data_properties,
            gmd_properties
        )
        if data_properties.min_tile_level:
            self.min_tile_level = \
                data_properties.min_tile_level
        if data_properties.max_tile_level:
            self.max_tile_level = \
                data_properties.max_tile_level



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-18


from gagospecs.v1.base.model import DataProperties, \
    GMDProperties, \
    ProductLevelValues, \
    ProductTypeValues, \
    FileTypeValues, \
    CoordinateSystemValues, \
    RasterDataTypeValues, \
    CompressionValues, \
    GeometryTypeValues, \
    BiasValues, \
    CategoryValues, \
    WeatherProductTypeValues
from gagospecs.v1.services.generate_gmd import \
    generate_from_gmd, \
    generate_from_data
from gagospecs.v1.services.properties import \
    get_data_properties, \
    get_data_properties_from_level_and_type, \
    get_gmd_properties, \
    get_gmd_properties_from_level_and_type
from gagospecs.v1.services.validate_data import \
    validate_data
from gagospecs.v1.services.validate_gmd import \
    validate_gmd



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-25


import json

from gagospecs.v1.base.load_model import load_model_from_gmd
from gagospecs.v1.base.model import DataProperties, \
    GMDProperties, \
    GMDModel, \
    create_model
from gagospecs.v1.base.schema import GMDSchema, \
    create_schema
from gagospecs.v1.services.validate_gmd import validate_gmd


def generate_from_data(
        product_level: str,
        product_type: str,
        data_properties: DataProperties,
        gmd_properties: GMDProperties
) -> (GMDModel, str, dict):
    """
    Generate a gmd model and str from data
    properties and gmd properties
    :param product_level:
    :param product_type:
    :param data_properties: Properties from data itself
    :param gmd_properties: Properties extra for gmd, name, description, etc
    :return: GMDModel, serialized GMDModel and errors
    """
    gmd_model: GMDModel = create_model(
        product_level,
        product_type
    )
    if gmd_model is None:
        return None, None, {
            'gmd': '指定产品级别或类型错误，无法创建元数据模型'
        }
    gmd_model.update(
        data_properties,
        gmd_properties
    )
    schema: GMDSchema = create_schema(
        product_level,
        product_type
    )
    gmd_str, errors = schema.dumps(gmd_model)
    if errors:
        return None, None, errors
    errors: dict = validate_gmd(
        gmd_str
    )
    if errors:
        return None, None, errors
    return \
        gmd_model, \
        gmd_str.encode('latin-1').decode('unicode_escape'), \
        {}


def generate_from_gmd(
        src_gmd: str or dict or GMDModel,
        dst_product_level: str,
        data_properties: DataProperties,
        gmd_properties: GMDProperties
) -> (GMDModel, str, dict):
    """
    Generate a gmd model from given source gmd model
    :param src_gmd:
    :param dst_product_level:
    :param data_properties:
    :param gmd_properties:
    :return: A tuple include GMDModel, serialized GMDModel and error dict
    """

    src_gmd_dict: dict = None
    if isinstance(src_gmd, str):
        try:
            src_gmd_dict = json.loads(
                src_gmd
            )
        except json.JSONDecodeError:
            return None, None, {
                'gmd内容': 'Gago Meta Data 内容错误'
            }

    if isinstance(src_gmd, dict):
        src_gmd_dict = src_gmd

    if src_gmd_dict:
        src_gmd_model, errors = \
            load_model_from_gmd(
                src_gmd_dict
            )
        if errors:
            return None, None, errors
    elif isinstance(src_gmd, GMDModel):
        src_gmd_model: GMDModel = src_gmd
    else:
        raise ValueError(
            'src_gmd is not str, dict or GMDModel'
        )

    product_type: str = src_gmd_model.product_type

    dst_gmd_model: GMDModel = create_model(
        dst_product_level,
        product_type
    )
    if not dst_gmd_model:
        return None, None, {
            'gmd内容': '无法根据GMD创建指定的元数据对象'
        }

    dst_gmd_model.copy_from(src_gmd_model)
    dst_gmd_model.update(
        data_properties,
        gmd_properties
    )

    dst_schema: GMDSchema = create_schema(
        dst_product_level,
        product_type
    )
    if not dst_schema:
        return None, None, {
            'gmd错误': '无法根据GMD验证指定的元数据对象'
        }
    dst_gmd_str, errors = dst_schema.dumps(dst_gmd_model)
    if errors:
        return None, None, errors
    errors: dict = validate_gmd(
        dst_gmd_str
    )
    if errors:
        return None, None, errors
    return \
        dst_gmd_model, \
        dst_gmd_str.encode('latin-1').decode('unicode_escape'), \
        {}



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-17

from gagospecs.v1 import ProductTypeValues, \
    CoordinateSystemValues, \
    RasterDataTypeValues
from gagospecs.v1.base.model import \
    FloatRasterGMDModel, \
    GeoTIFFRasterGMDModel
from gagospecs.v1.products.level2.models.general import Level2RasterGMDModel


class NDVIModel(
    GeoTIFFRasterGMDModel,
    FloatRasterGMDModel,
    Level2RasterGMDModel
):
    def __init__(self):
        super(NDVIModel, self).__init__()
        self.name = 'NDVI'
        self.description = 'NDVI'
        self.product_type = ProductTypeValues.NDVI
        self.coordinate_system = CoordinateSystemValues.EPSG_4326
        self.data_type = RasterDataTypeValues.Byte
        self.no_data = 255
        self.linear_factor = -127
        self.scale_factor = 0.0078125


def main():
    return NDVIModel()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-25


from gagospecs.v1.base.model import RasterGMDModel, \
    DataProperties, \
    GMDProperties, \
    FileTypeValues, \
    ProductTypeValues, \
    RasterDataTypeValues, \
    CoordinateSystemValues
from gagospecs.v1.products.level3.models.general \
    import Level3RasterModel


class CropClassificationModel(
    RasterGMDModel,
    Level3RasterModel
):
    def __init__(self):
        super(CropClassificationModel, self).__init__()
        self.name = 'CropClassification'
        self.description = 'CropClassification'
        self.product_type = ProductTypeValues.CropClassification
        self.file_type = FileTypeValues.LERC
        self.coordinate_system = CoordinateSystemValues.EPSG_4326
        self.data_type = RasterDataTypeValues.Byte
        self.no_data = 255
        self.classification = dict()

    def _validate(
            self,
            data_properties: DataProperties,
            errors: list
    ):
        super()._validate(
            data_properties,
            errors
        )
        if not data_properties.data_value_list:
            errors.append('classification/data_value_list not set')
            return
        if len(self.classification) != \
                len(data_properties.data_value_list):
            errors.append('classification')
        else:
            for v in data_properties.data_value_list:
                if v not in self.classification:
                    errors.append('classification')
                    break

    def update(
            self,
            data_properties: DataProperties,
            gmd_properties: GMDProperties
    ):
        super().update(
            data_properties,
            gmd_properties
        )
        self.classification = data_properties.classification


def main():
    return CropClassificationModel()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-08


import json
from datetime import datetime

from gagospecs.v1.base.load_module import create_module


class DataProperties:
    """
    用于检验数据本体属性与元数据的集合
    """
    def __init__(self):
        # file type, FileTypeValues
        self.file_type = None
        # RasterDataTypeValues
        self.data_type: str = None
        # no data value
        self.no_data: int = None
        # line count
        self.columns: int = None
        # row count
        self.rows: int = None
        # resolution of image
        self.resolution: float = None
        # CompressionValues, NA, LZW
        self.compression: str = None
        # 坐标偏移
        self.bias: str = None
        # 'EPSG:4326' or 'EPSG:3587'....
        self.coordinate_system: str = None
        # [[x1, y1], [x1, y2], [x2, y2], [x2, y1]]
        self.extent: str = None
        # enum of data values [1, 3, 5, 6]
        # for crop classification like product
        self.data_value_list: list = None
        # min value, for continuous product
        self.data_min: int = None
        # max value, for continuous product
        self.data_max: int = None
        # GeometryTypeValues, for Vector Product
        self.geometry_type: str = None
        # Min tile level
        self.min_tile_level: int = None
        # Max tile level
        self.max_tile_level: int = None
        # classification information
        self.classification: dict = None

    def need_data_value_list(self) -> bool:
        return False


class GMDProperties:
    def __init__(self):
        # product name
        self.name: str = None
        # product description
        self.description: str = None
        # QC, will append to QC
        self.qc: str = None
        # Production datetime
        self.production_datetime: datetime = None
        # Producer, will append to producer
        self.producer: str = None


class GMDModel:
    def __init__(self):
        self.version: str = '1.0'
        self.name: str = ''
        self.description: str = ''
        self.level: str = ''
        self.category: str = ''
        self.product_type: str = ''
        self.file_type: str = ''
        self.production_datetime: datetime = None
        self.producer: str = ''
        self.qc: str = ''

    def validate(
            self,
            data_properties: DataProperties
    ) -> list:
        validation_errors: list = []
        self._validate(
            data_properties,
            validation_errors
        )
        return validation_errors

    def _validate(
            self,
            data_properties: DataProperties,
            errors: list
    ):
        if self.file_type != data_properties.file_type:
            errors.append('file_type')

    def update(
            self,
            data_properties: DataProperties,
            gmd_properties: GMDProperties
    ):
        if gmd_properties.name:
            self.name = gmd_properties.name
        if gmd_properties.description:
            self.description = gmd_properties.description
        if gmd_properties.production_datetime:
            self.production_datetime = gmd_properties.production_datetime
        if gmd_properties.producer:
            producer_list = self.producer.split('#') \
                if self.producer \
                else list()
            producer_list.append(gmd_properties.producer)
            self.producer = '#'.join(producer_list)
        if gmd_properties.qc:
            qc_list = self.qc.split('#') \
                if self.qc \
                else list()
            qc_list.append(gmd_properties.qc)
            self.qc = '#'.join(qc_list)

    def copy_from(
            self,
            gmd_model
    ):
        self.producer = getattr(gmd_model, 'producer', self.producer)
        self.qc = getattr(gmd_model, 'qc', self.qc)


class SpatialGMDModel(GMDModel):
    def __init__(self):
        super().__init__()
        self.coordinate_system: str = ''
        self.extent: str = ''
        self.bias: str = 'NA'

    def _validate(
            self,
            data_properties: DataProperties,
            errors: list
    ):
        super()._validate(
            data_properties,
            errors
        )
        if self.coordinate_system != data_properties.coordinate_system:
            errors.append('coordinate_system')
        if isinstance(data_properties.extent, str):
            extent_str: str = data_properties.extent
        else:
            extent_str: str = json.dumps(data_properties.extent)
        if self.extent != extent_str:
            errors.append('extent')
        if self.bias != data_properties.bias:
            errors.append('bias')

    def update(
            self,
            data_properties: DataProperties,
            gmd_properties: GMDProperties
    ):
        super().update(
            data_properties,
            gmd_properties
        )
        if data_properties.coordinate_system:
            self.coordinate_system = \
                data_properties.coordinate_system
        if data_properties.extent:
            self.extent = data_properties.extent
        if data_properties.bias:
            self.bias = data_properties.bias

    def copy_from(
            self,
            gmd_model: GMDModel
    ):
        super().copy_from(gmd_model)
        self.coordinate_system = getattr(
            gmd_model,
            'coordinate_system',
            self.coordinate_system
        )
        self.extent = getattr(
            gmd_model,
            'extent',
            self.extent
        )
        self.bias = getattr(
            gmd_model,
            'bias',
            self.bias
        )


class RasterGMDModel(SpatialGMDModel):
    def __init__(self):
        super().__init__()
        self.category = CategoryValues.Raster
        self.no_data: int = -1
        self.data_type: str = None

    def _validate(
            self,
            data_properties: DataProperties,
            errors: list
    ):
        super()._validate(
            data_properties,
            errors
        )
        if self.no_data != data_properties.no_data:
            errors.append('no_data')
        if self.data_type != data_properties.data_type:
            errors.append('data_type')

    def update(
            self,
            data_properties: DataProperties,
            gmd_properties: GMDProperties
    ):
        super().update(
            data_properties,
            gmd_properties
        )
        if data_properties.no_data:
            self.no_data = data_properties.no_data
        if data_properties.data_type:
            self.data_type = data_properties.data_type

    def copy_from(
            self,
            gmd_model: GMDModel
    ):
        super().copy_from(
            gmd_model
        )
        self.no_data = getattr(
            gmd_model,
            'no_data',
            self.no_data
        )
        self.data_type = getattr(
            gmd_model,
            'data_type',
            self.data_type
        )


class FloatRasterGMDModel(RasterGMDModel):
    def __init__(self):
        super().__init__()
        self.scale_factor: float = 1.0
        self.linear_factor: float = 1.0

    def copy_from(
            self,
            gmd_model: GMDModel
    ):
        super().copy_from(gmd_model)
        self.scale_factor = getattr(
            gmd_model,
            'scale_factor',
            self.scale_factor
        )
        self.linear_factor = getattr(
            gmd_model,
            'linear_factor',
            self.linear_factor
        )


class GeoTIFFRasterGMDModel(GMDModel):
    def __init__(self):
        super().__init__()
        self.file_type: str = FileTypeValues.GeoTIFF
        self.compression: str = CompressionValues.LZW

    def _validate(
            self,
            data_properties: DataProperties,
            errors: list
    ):
        super()._validate(
            data_properties,
            errors
        )
        if self.compression != data_properties.compression:
            errors.append('compression')

    def update(
            self,
            data_properties: DataProperties,
            gmd_properties: GMDProperties
    ):
        super().update(
            data_properties,
            gmd_properties
        )
        if data_properties.compression:
            self.compression = data_properties.compression


def create_model(
        product_level: str,
        product_type: str
) -> GMDModel or None:
    """
    Create a GMD model
    :param product_level: product level,
    which laid in gagospecs.v1.base.product_level.ProductLevel class
    :param product_type: product name
    :return: requested GMD model object
    """
    m = create_module(
        'models',
        product_level,
        product_type
    )
    if m:
        if hasattr(m, 'main'):
            return getattr(m, 'main')()
    return None


def create_data_properties(
        product_level: str,
        product_type: str
) -> DataProperties or None:
    """
    Get DataProperies object needed
    for the specified product
    :param product_level:
    :param product_type:
    :return:
    """
    m = create_module(
        'models',
        product_level,
        product_type
    )
    if m:
        if hasattr(m, 'main_properties'):
            return m.main_properties()
    return DataProperties()


def create_gmd_properties(
        product_level: str,
        product_type: str
) -> GMDProperties or None:
    """
    Get GMDProperties object needed
    for the specified product
    :param product_level:
    :param product_type:
    :return:
    """
    m = create_module(
        'models',
        product_level,
        product_type
    )
    if m:
        if hasattr(m, 'main_gmd_properties'):
            return m.main_gmd_properties()
    return GMDProperties()


class ValuesBase:
    """
    值域表基类
    """
    @classmethod
    def get_value_set(cls) -> set:
        values_set: set = set()
        for k, v in cls.__dict__.items():
            key: str = k
            value: str = v
            if (not key.startswith('__')
                and value not in values_set):
                values_set.add(value)
        return values_set


class ProductLevelValues(ValuesBase):
    """
    Product level
    """
    Level1 = 'level1'
    Level2 = 'level2'
    Level3 = 'level3'


class CategoryValues(ValuesBase):
    """
    产品分类值域表
    """
    Raster = 'Raster'
    Vector = 'Vector'
    Attribute = 'Attribute'


class ProductTypeValues(ValuesBase):
    """
    产品类型值域表
    """
    BaseMap = 'BaseMap'
    NDVI = 'NDVI'
    CropClassification = 'CropClassification'
    DisasterAssessment = 'DisasterAssessment'
    PDSI = 'PDSI'
    SMAP = 'SMAP'
    AdministrativeArea = 'AdministrativeArea'
    LandBlock = 'LandBlock'
    Statistics = 'Statistics'
    WeatherLive = 'WeatherLive'
    WeatherHistory = 'WeatherHistory'
    WeatherForecast = 'WeatherForecast'


class WeatherProductTypeValues(ValuesBase):
    """
    气象产品类型值域表
    """
    NA = 'NA'
    WeatherPhenomenon = 'WeatherPhenomenon'
    Temperature = 'Temperature'
    Moisture = 'Moisture'
    Precipitation = 'Precipitation'
    WindDirection = 'WindDirection'
    WindPower = 'WindPower'
    Pressure = 'Pressure'
    AirQuality = 'AirQuality'


class FileTypeValues(ValuesBase):
    """
    文件类型值域表
    """
    GeoTIFF = 'GeoTIFF'
    JPEG = 'JPEG'
    PNG = 'PNG'
    LERC = 'LERC'
    BIP = 'BIP'
    GeoJSON = 'GeoJSON'
    SHP = 'SHP'
    JSON = 'JSON'


class CoordinateSystemValues(ValuesBase):
    """
    坐标系统值域表
    """
    EPSG_3857 = 'EPSG:3857'
    EPSG_4326 = 'EPSG:4326'


class BiasValues(ValuesBase):
    """
    坐标系偏移值域表
    """
    NA = 'NA'
    GCJ_02 = 'GCJ-02'


class RasterDataTypeValues(ValuesBase):
    """
    栅格像元数据类型值域表
    """
    Byte = 'Byte'
    UInt16 = 'UInt16'
    UInt32 = 'UInt32'


class CompressionValues(ValuesBase):
    """
    栅格数据压缩方法值域表
    """
    NA = 'NA'
    LZW = 'LZW'


class GeometryTypeValues(ValuesBase):
    """
    几何类型值域表
    """
    Point = 'Point'
    Polyline = 'Polyline'
    Polygon = 'Polygon'



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-25


from marshmallow import ValidationError, \
    validates, \
    validates_schema
from marshmallow.fields import Integer

from gagospecs.v1.base.model import ProductLevelValues
from gagospecs.v1.base.schema import GMDSchema


class Level3GMDSchema(GMDSchema):
    """
    三级产品模式
    """
    @validates('level')
    def validate_level(self, value):
        if value != ProductLevelValues.Level3:
            raise ValidationError(
                '三级产品level字段必须为level3'
            )


class Level3RasterSchema(Level3GMDSchema):
    """
    三级栅格GMD模式
    """
    min_tile_level = Integer(required=True, allow_none=False)
    max_tile_level = Integer(required=True, allow_none=False)

    @validates('min_tile_level')
    def validate_min_tile_level(self, value):
        if value < 0 or value > 20:
            raise ValidationError(
                'min_tile_level字段必须大于等于0且小于等于20'
            )

    @validates('max_tile_level')
    def validate_max_tile_level(self, value):
        if value < 0 or value > 20:
            raise ValidationError(
                'max_tile_level字段必须大于等于0且小于等于20'
            )

    @validates_schema
    def validate_tile_levels(self, data):
        if data['min_tile_level'] > data['max_tile_level']:
            raise ValidationError(
                'min_tile_level字段不能大于max_tile_level字段'
            )



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-25


from marshmallow import validates, \
    ValidationError, \
    post_load

from gagospecs.v1.base.model import FileTypeValues, \
    ProductTypeValues, \
    RasterDataTypeValues
from gagospecs.v1.base.schema import FloatRasterGMDSchema
from gagospecs.v1.base.schema import post_load_object
from gagospecs.v1.products.level3.models.NDVI import NDVIModel
from gagospecs.v1.products.level3.schemas.general import Level3RasterSchema


class NDVISchema(
    FloatRasterGMDSchema,
    Level3RasterSchema
):
    @validates('file_type')
    def validate_file_type(self, value):
        if value != FileTypeValues.LERC:
            raise ValidationError('三级NDVI产品file_type字段必须为LERC')

    @validates('product_type')
    def validate_product_type(self, value):
        if value != ProductTypeValues.NDVI:
            raise ValidationError('三级NDVI栅格产品的product_type字段必须为"NDVI"')

    @validates('data_type')
    def validate_data_type(self, value):
        if value != RasterDataTypeValues.Byte:
            raise ValidationError('三级NDVI栅格产品data_type字段必须为"Byte"')

    @validates('no_data')
    def validate_no_data(self, value):
        if value != 255:
            raise ValidationError('三级NDVI栅格产品no_data字段必须为"255"')

    @validates('scale_factor')
    def validate_scale_factor(self, value):
        if value != 0.0078125:
            raise ValidationError('三级NDVI栅格产品scale_factor字段必须为"0.0078125"')

    @validates('linear_factor')
    def validate_linear_factor(self, value):
        if value != -127:
            raise ValidationError('三级NDVI栅格产品linear_factor字段必须为"-127"')

    @post_load
    def make_object(self, data: dict):
        return post_load_object(NDVIModel(), **data)


def main():
    return NDVISchema()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-17


from sys import modules


def create_module(
        name_space: str,
        product_level: str,
        product_type: str
):
    try:
        module_name: str = 'gagospecs.v1.products.' + \
                           product_level + \
                           '.' + \
                           name_space + \
                           '.' + \
                           product_type
        __import__(module_name)
        m = modules[module_name]
    except ImportError:
        m = None

    return m



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-10-18


from distutils.core import setup

setup(
    name='gagospecs',
    version='1.0.0',
    description='Gago Specs Lib',
    author='Dajiang Ren',
    author_email='rendajiang@gagogroup.cn',
    url='http://www.gagogroup.cn',
    packages=[
        'gagospecs',
        'gagospecs.base',
        'gagospecs.v1',
        'gagospecs.v1.base',
        'gagospecs.v1.products.level2.converters',
        'gagospecs.v1.products.level2.models',
        'gagospecs.v1.products.level2.schemas',
        'gagospecs.v1.products.level3.models',
        'gagospecs.v1.products.level3.schemas',
        'gagospecs.v1.services',
    ],
    requires=[
        'gdal',
        'marshmallow'
    ],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 


from marshmallow import validates, \
    ValidationError, \
    post_load
from marshmallow.fields import Dict, \
    Integer, \
    String

from gagospecs.v1 import ProductTypeValues, \
    FileTypeValues, \
    RasterDataTypeValues, \
    CompressionValues
from gagospecs.v1.base.schema import RasterGMDSchema
from gagospecs.v1.base.schema import post_load_object
from gagospecs.v1.products.level3.models.CropClassification \
    import CropClassificationModel
from gagospecs.v1.products.level3.schemas.general import Level3RasterSchema


class CropClassificationSchema(
    RasterGMDSchema,
    Level3RasterSchema
):
    """
    Crop classification schema
    """
    classification = Dict(keys=Integer(), values=String(), required=True)

    @validates('file_type')
    def validate_file_type(self, value):
        if value != FileTypeValues.LERC:
            raise ValidationError('作物分类栅格产品必须为LERC格式')

    @validates('product_type')
    def validate_product_type(self, value):
        if value != ProductTypeValues.CropClassification:
            raise ValidationError(
                '作物分类栅格产品的product_type字段必须为"CropClassification"'
            )

    @validates('data_type')
    def validate_data_type(self, value):
        if value != RasterDataTypeValues.Byte:
            raise ValidationError('作物分类栅格产品data_type字段必须为"Byte"')

    @validates('no_data')
    def validate_no_data(self, value):
        if value != 255:
            raise ValidationError('作物分类栅格产品no_data字段必须为"255"')

    @validates('classification')
    def validate_classification(self, value):
        value: dict = value
        for k, v in value.items():
            try:
                k_int: int = int(k)
            except ValueError:
                raise ValidationError('classification分类值中像元值不为整型')
            if k_int < 0 or k_int > 254:
                raise ValidationError('classification分类值中像元值不合法，应为0到254之间（含）')
            if not isinstance(v, str):
                raise ValidationError('classification分类值作物名称错误')

    @post_load
    def make_object(self, data: dict):
        return post_load_object(CropClassificationModel(), **data)


def main():
    return CropClassificationSchema()